let currentCursor;
let currentComponent;
export function hooks(config) {
    const h = currentComponent.hooks;
    const index = currentCursor++;
    if (h.values.length <= index && config.oncreate) {
        h.values[index] = config.oncreate(h, currentComponent, index);
    }
    if (config.onupdate) {
        h.values[index] = config.onupdate(h, currentComponent, index);
    }
    return h.values[index];
}
export class Component extends HTMLElement {
    constructor() {
        super(...arguments);
        this.updating = false;
        this.$root = this.attachShadow({ mode: "open" });
        this.hooks = {
            values: [],
            deps: [],
            effects: [],
            cleanup: []
        };
    }
    connectedCallback() {
        this.update();
    }
    disconnectedCallback() {
        this.hooks.cleanup.forEach(f => f());
    }
    update() {
        if (this.updating)
            return;
        this.updating = true;
        Promise.resolve().then(() => {
            try {
                currentCursor = 0;
                currentComponent = this;
                this.render();
                this.affect();
            }
            catch (e) {
                console.error(e);
            }
            this.updating = false;
        });
    }
    affect() {
        const h = this.hooks;
        for (let i = 0; i < h.effects.length; i++) {
            if (h.effects[i]) {
                h.cleanup[i] && h.cleanup[i]();
                const cleanup = h.effects[i]();
                if (cleanup) {
                    h.cleanup[i] = cleanup;
                }
                delete h.effects[i];
            }
        }
    }
}
//# sourceMappingURL=component.js.map